#include "common.h"
#include "debug_io.h"
#include "write_typed_value.h"
#include "communication.h"
#include "write_communication.h"

#define debug(fun_name, msg) pio_debug (__FILE__ "::" fun_name, msg)

/* tv_t values[]; might be tv_t **values XXXX */
static void w_values (pio_buffer *ob, size_t nvals, TV_t *values) {
  debug("w_values", "start writing");
  size_t i = 0;

  for (i = 0; i < nvals; i++)
    write_typed_value (ob, &values[i]);

  debug("w_values", "finish writing");
}

static void d_com_msg (pio_buffer *msg, const COM_t *com) {
  switch (com->com_tag) {
  case COM_Phrase:	buf_add_string (msg, "Phrase");				break;
  case COM_Task:	buf_printf (msg, "Task \"%s\"", com->vs.task.t_name);	break;
  case COM_Result:	buf_add_string (msg, "Result");				break;
  case COM_Error:	buf_add_string (msg, "Error");				break;
  case COM_Service:	buf_printf (msg,
				    "Service \"%s\"", string_of_service_name (com->vs.svc.srv_name));
										break;
  default:
    assert (0);
  }
  buf_printf (msg, " of length %d", com->nvalues);
}

static void write_com (pio_buffer *ob, const COM_t *com) {
#ifdef DO_DEBUG
  pio_buffer msg;
  buf_create (&msg, 10);
  buf_add_string (&msg, "start writing ");
  d_com_msg (&msg, com);
  debug ("write_com", msg.buf_fer);
  free (msg.buf_fer);
#endif

  switch (com->com_tag) {
  case COM_Phrase:
    buf_printf (ob, "%%p <%d>\n", com->nvalues);
    w_values (ob, com->nvalues, com->vs.phrase_values);
    break;
  case COM_Task:
    buf_printf (ob, "%%t <%d> \"%s\"\n", com->nvalues, com->vs.task.t_name);
    w_values (ob, com->nvalues, com->vs.task.t_values);
    break;
  case COM_Result:
    buf_printf (ob, "%%r <%d>\n", com->nvalues);
    w_values (ob, com->nvalues, com->vs.result_values);
    break;
  case COM_Error:
    buf_printf (ob, "%%e <%d>\n", com->nvalues);
    w_values (ob, com->nvalues, com->vs.error_values);
    break;
  case COM_Service:
    buf_printf (ob, "%%s <%d> \"%s\"\n", com->nvalues,
		string_of_service_name (com->vs.svc.srv_name));
    w_values (ob, com->nvalues, com->vs.svc.srv_values);
    break;
  }
  debug ("write_com", "finish writing");
}

void write_communication (pio_buffer *ob, const COM_t *comm) {
  buf_add_string (ob, "(\n");
  write_com (ob, comm);
  buf_add_string (ob, ")\n\n");
}
